library attribute_directives.shorten_overflow_text;

import 'dart:html' as html;
import 'dart:math';
import 'package:angular2/angular2.dart';

@Directive(selector: '[foShortenOverflow]')
class ShortenOverflowTextDirective implements OnInit
{
  ShortenOverflowTextDirective(this._elementRef);

  void ngOnInit()
  {
    html.window.onResize.listen((_) => _adjustWidth(_elementRef.nativeElement.text));
  }

  void _adjustWidth(String value)
  {
    html.Element nativeElement = _elementRef.nativeElement;

    if (value == null) value = nativeElement.text;

    nativeElement.text = value;

    html.Rectangle<double> parentRect = nativeElement.parent.getBoundingClientRect();
    html.Rectangle<double> rect = nativeElement.getBoundingClientRect();
    double parentWidth = parentRect.right - parentRect.left;
    double width = rect.right - rect.left;
    double dotMargin = 20.0;
    int overflowGuard = 999;

    while (width + dotMargin > parentWidth && overflowGuard > 0)
    {
      value = value.substring(0, max(value.length - 1, 0));
      nativeElement.text = value;
      rect = nativeElement.getBoundingClientRect();
      width = rect.right - rect.left;
      overflowGuard--;
    }
    if (overflowGuard < 999) nativeElement.text += "...";
  }

  @Input('foShortenOverflow')
  void set text(String value)
  {
    _adjustWidth(value);
  }

  /*
  @HostListener('resize')
  void onResize()
  {
    print("RESIZE");
    _adjustWidth(null);
  }
  */

  final ElementRef _elementRef;
}